{
  Bin2Pas v1.0
  Convierte cualquier archivo a su representacion binaria en un arreglo del lenguaje Pascal.
  Copyright (C) 2003  Hctor Mauricio Rodrguez Segura

  WWW:   http://www.geocities.com/hmsoftcr
  EMail: ranametal@blistering.net

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
program Bin2Pas;
{$APPTYPE CONSOLE}
uses
  Windows,
  SysUtils,
  Classes;

type
  TBin2PasResult = (rOk, rInvalidToken, rInvalidUnitName, rUnknow);

var
  MsgStr: String;

const
  SComment = '{'#13#10 +
    '  Archivo auto generado por Bin2Pas de Hector Mauricio Rodriguez el %s'#13#10 +
    '  a las %s como resultado de procesar el archivo "%s"'#13#10'}'#13#10;

  SInterface = 'unit %s;'#13#10#13#10 +
               'interface'#13#10#13#10 +
               'const'#13#10 +
               '  Max%s = %d; {Tamao del arreglo}'#13#10#13#10 +
               'type'#13#10 +
               '  P%s = ^T%s; {Puntero al arreglo}'#13#10 +
               '  T%s = array[1..Max%s] of Byte; {Definicin del tipo de arreglo}'#13#10#13#10 +
               'var'#13#10 +
               '  %s: T%s = {Datos del arreglo}'#13#10'  (';

  SImplementation = 'implementation'#13#10#13#10 +
                    'end.';

function BinToPas(SrcFile, DestFile, Token: String): TBin2PasResult;
var
  DestStream, SrcStream: TMemoryStream;
  P: PByte;
  S, UName: String;
  B: Byte;
  C, I: Integer;
begin
  try
    SrcStream := TMemoryStream.Create;
    try
      SrcStream.LoadFromFile(SrcFile);
      DestStream := TMemoryStream.Create;
      try
        UName :=  ExtractFileName(DestFile);
        Delete(UName, Pos('.', UName), MaxInt);
        if not IsValidIdent(UName) then
        begin
          Result := rInvalidUnitName;
          Abort;
        end;

        if not IsValidIdent(Token) then
        begin
          Result := rInvalidToken;
          Abort;
        end;

        S := Format(SComment, [DateToStr(Date), TimeToStr(Time), SrcFile]);
        DestStream.Write(PChar(S)^, Length(S));

        S := Format(SInterface, [UName, Token, SrcStream.Size, Token, Token,
          Token, Token, Token, Token]);
        DestStream.Write(PChar(S)^, Length(S));

        C := 0;
        P := SrcStream.Memory;
        for I := 1 to SrcStream.Size do
        begin
          B := P^; Inc(P); Inc(C);
          S := IntToStr(B);
          if I = SrcStream.Size then
             S := S + ');'#13#10#13#10
          else if C = 25 then
          begin
            S := S + ','#13#10'  ';
            C := 0;
          end else
          S := S + ',';
          DestStream.Write(PChar(S)^, Length(S));
        end;

        DestStream.Write(PChar(SImplementation)^, Length(SImplementation));

        if ExtractFileExt(DestFile) = '' then
          DestFile := DestFile + '.pas';
        DestStream.SaveToFile(DestFile);
      finally
        DestStream.Free;
      end;
    finally
      SrcStream.Free
    end;
    Result := rOk;
  except
    on E: Exception do
    begin
      if not (E is EAbort) then
      begin
        Result := rUnknow;
        MsgStr := E.Message;
      end;
    end;
  end;
end;

procedure PrintHeader;
begin
  Writeln;
  Writeln('Bin2Pas - (c)2003 Hector Mauricio Rodriguez Segura');
  Writeln;
end;

procedure PrintHelp;
begin
  Writeln('Convierte cualquier archivo a su representacion binaria en un arreglo');
  Writeln('del lenguaje Pascal.'); Writeln;
  Writeln('Uso:');
  Writeln('Bin2Pas <BinFileName> <PasFileName> <TokenName>');
  Writeln('Bin2Pas C:\Command.com C:\Command.pas Cmd');
end;


var
  Resultado: TBin2PasResult;
begin
  PrintHeader;
  if ParamCount = 3 then
  begin
    Resultado := BinToPas(ParamStr(1), ParamStr(2), ParamStr(3));
    case Resultado of
      rOk: Writeln('Finalizado correctamente!!!');
      rInvalidUnitName: Writeln(Format('Error: "%s" no es un nombre de archivo valido.', [ParamStr(2)]));
      rInvalidToken: Writeln(Format('Error: "%s" no es un identificador valido.', [ParamStr(3)]));
      rUnknow: Writeln('Error: ', MsgStr);
    end;
    Halt(Ord(Resultado));
  end else PrintHelp;
end.
